<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01004PratosElaboracaoCardapioBase extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $retornos = array();
    protected $columnToReturn;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService   = $crudService;
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function retornaMascaraNivelPrato(DTO\Request $request, DTO\Response $response) {
        $results = $this->mascaraNivelPrato();
        $response->addDataSet(new DataSet("mascaraNivelPrato", $results));
    }
    
    private function mascaraNivelPrato() {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT CDPICTPRAT,
                       NRORG
                  FROM PARAMGERAL
                 WHERE NRORG = :NRORG";
                 
        $sql1 = "SELECT MAX(CDNVPRATO) AS CDNVPRATO,
                        NRORG
                   FROM PRATO
                  WHERE NRORG = :NRORG
                  GROUP BY NRORG";
        
        $query = $connection->prepare($sql);
        $query1 = $connection->prepare($sql1);
        
        $params = array(
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        $params1 = array(
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        $query1->execute($params1);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        $result1 = $query1->fetch(\PDO::FETCH_ASSOC);
        
        $results = array(
            "CDPICTPRAT" => $result["CDPICTPRAT"],    
            "CDNVPRATO" => $result1["CDNVPRATO"]    
        );
        
        return $results;
    }
    
    public function retornaReceitas(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $response->addDataSet(new DataSet("receitaPrato", $this->produtosReceita($row))); 
    }
    
    private function determinaReceitaUtilizada($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT RECE_UTIL_TCO(:CDFILAUXPRAT, 
                                     :CDPRATO, 
                                     (SELECT CDFILIAL FROM PARAMTCO WHERE NRORG = :NRORG), 
                                     :NRSERVICOPROPOSTA,
                                     :NRPADRAOQUALIDADE, 
                                     :CDREGIAO,
                                     :NRCARDAPIOTRAB) AS IDRECEITA
                  FROM DUAL";
                  
        $params = array(
            "CDFILAUXPRAT"      => $row["CDFILAUXPRAT"],
            "CDPRATO"           => $row["CDPRATO"],
            "NRORG"             => $this->environment->getNrOrgTrab(),
            "NRSERVICOPROPOSTA" => $row["NRSERVICOPROPOSTA"],
            "NRPADRAOQUALIDADE" => $row["NRPADRAOQUALIDADE"],
            "CDREGIAO"          => $row["CDREGIAO"],
            "NRCARDAPIOTRAB"    => $row["NRCARDAPIOTRAB"]
        );
                   
        $query = $connection->prepare($sql);
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["IDRECEITA"];
    }
    
    private function produtosReceita($row) {
        $connection = $this->entityManager->getConnection();
        
        $consultas = array(
            // Padrão
            "P" => "SELECT 'P' AS TIPORECE, PR.CDARVPROD, PR.NMPRODUTO, RP.QTPRODPRAT AS QTPRODUTO, NVL( PR.SGUNIDADE, PR.SGUNIDADE ) AS SGUNIDADE, RP.VRPEFATCOCPR AS VRGANHOCOCCAO, ( RP.VRPEFATCOCPR + 100 ) AS VRAPCOCCAO, RP.VRPEFATCORPR AS VRPERDALIMPEZA, ( 100 - RP.VRPEFATCORPR ) AS VRAPLIMPEZA, RP.IDPRODBASEPR, RP.QTBRUTAPRUNR AS QTDBRUTA, RP.IDQTDEFIXPR FROM RECEPADR RP INNER JOIN PRODUTO PR ON PR.CDPRODUTO = RP.CDPRODUTO WHERE RP.CDPRATO =:CDPRATO AND RP.CDFILAUXPRAT =:CDFILAUXPRAT ORDER BY PR.CDARVPROD",
            // Local
            "F" => "SELECT 'F' AS TIPORECE, PD.CDARVPROD, PD.NMPRODUTO, RF.QTPRODRECLOC AS QTPRODUTO, NVL( RF.SGUNIDADE, PD.SGUNIDADE ) AS SGUNIDADE, RF.VRPEFATCOCLO AS VRGANHOCOCCAO, ( RF.VRPEFATCOCLO + 100 ) AS VRAPCOCCAO, RF.VRPEFATCORLO AS VRPERDALIMPEZA, ( 100 - RF.VRPEFATCORLO ) AS VRAPLIMPEZA, RF.QTBRUTALOUNR AS QTDBRUTA, RF.IDPRODBASELO, RF.IDQTDEFIXLO, RF.CDFILAUXPRAT, RF.CDPRATO, RF.CDFILIAL, RF.CDPRODUTO FROM RECEFILI RF INNER JOIN PRODUTO PD ON PD.CDPRODUTO = RF.CDPRODUTO WHERE RF.CDFILAUXPRAT = :CDFILAUXPRAT AND RF.CDPRATO = :CDPRATO AND RF.CDFILIAL = ( SELECT CDFILIAL FROM PARAMTCO WHERE NRORG = :NRORG ) ORDER BY PD.CDARVPROD",
            // Padrão de Qualidade
            "Q" => "SELECT 'Q' AS TIPORECE, P.CDARVPROD, P.NMPRODUTO, R.QTPRODRECPADQ AS QTPRODUTO, NVL( R.SGUNIDADE, P.SGUNIDADE ) AS SGUNIDADE, R.VRPEFATCOCPADQ AS VRGANHOCOCCAO, (R.VRPEFATCOCPADQ + 100) AS VRAPCOCCAO, R.VRPEFATCORPADQ AS VRPERDALIMPEZA, (100 - R.VRPEFATCORPADQ) AS VRAPLIMPEZA, R.QTBRUTAPADQUNR AS QTDBRUTA, R.IDPRODBASEPADQ, R.IDQTDEFIXPADQ FROM RECEPADQUALI R INNER JOIN PRODUTO P ON P.CDPRODUTO = R.CDPRODUTO WHERE R.CDFILAUXPRAT = :CDFILAUXPRAT AND R.CDPRATO = :CDPRATO AND R.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE ORDER BY P.CDARVPROD",
            // Região
            "R" => "SELECT 'R' AS TIPORECE, P.CDARVPROD, P.NMPRODUTO, R.QTPRODRECREGIAO AS QTPRODUTO, NVL( R.SGUNIDADE, P.SGUNIDADE ) AS SGUNIDADE, R.QTPRODRECREGIAO, R.VRPEFATCOCREGIAO AS VRGANHOCOCCAO, (R.VRPEFATCOCREGIAO + 100) AS VRAPCOCCAO, R.VRPEFATCORREGIAO AS VRPERDALIMPEZA, (100 - R.VRPEFATCORREGIAO) AS VRAPLIMPEZA, R.QTBRUTAREGIAOUNR AS QTDBRUTA, R.IDPRODBASEREGIAO, R.IDQTDEFIXREGIAO FROM RECEREGIAO R INNER JOIN PRODUTO P ON P.CDPRODUTO = R.CDPRODUTO WHERE R.CDFILAUXPRAT = :CDFILAUXPRAT AND R.CDPRATO = :CDPRATO AND R.CDREGIAO = :CDREGIAO ORDER BY P.CDARVPROD",
            // Serviço
            "S" => "SELECT 'S' AS TIPORECE, PD.CDARVPROD, PD.NMPRODUTO, RS.QTPRODRECSER AS QTPRODUTO, NVL( RS.SGUNIDADE, PD.SGUNIDADE ) AS SGUNIDADE, RS.VRPEFATCOCSE AS VRGANHOCOCCAO, ( RS.VRPEFATCOCSE + 100 ) AS VRAPCOCCAO, RS.VRPEFATCORSE AS VRPERDALIMPEZA, ( 100 - RS.VRPEFATCORSE ) AS VRAPLIMPEZA, RS.QTBRUTASEUNR AS QTDBRUTA, RS.IDPRODBASESE, RS.IDQTDEFIXSE, RS.CDFILAUXPRAT, RS.CDPRATO, RS.CDFILIAL, RS.CDSERVFILI, RS.CDPRODUTO FROM RECESERV RS, PRODUTO PD WHERE RS.CDPRATO = :CDPRATO AND RS.CDFILAUXPRAT = :CDFILAUXPRAT AND RS.CDFILIAL = ( SELECT CDFILIAL FROM PARAMTCO WHERE NRORG = :NRORG ) AND RS.CDSERVFILI = :CDSERVFILI AND PD.CDPRODUTO = RS.CDPRODUTO ORDER BY PD.CDARVPROD",
            // Proposta
            "C" => "SELECT 'C' AS TIPORECE, PD.CDARVPROD, PD.NMPRODUTO, RC.QTPRODRECCARDTRAB AS QTPRODUTO, NVL( RC.SGUNIDADE, PD.SGUNIDADE ) AS SGUNIDADE, RC.VRPEFATCOCCARDTRAB AS VRGANHOCOCCAO, ( RC.VRPEFATCOCCARDTRAB + 100 ) AS VRAPCOCCAO, RC.VRPEFATCORCARDTRAB AS VRPERDALIMPEZA, ( 100 - RC.VRPEFATCORCARDTRAB ) AS VRAPLIMPEZA, RC.QTBRUTACARDTRABUNR AS QTDBRUTA, RC.IDPRODBASECARDTRAB, RC.IDQTDEFIXCARDTRAB, RC.CDFILAUXPRAT, RC.CDPRATO, RC.CDPRODUTO, RC.NRCARDAPIOTRAB FROM RECEPRATCARDTRAB RC, PRODUTO PD WHERE RC.CDPRATO = :CDPRATO AND RC.CDFILAUXPRAT = :CDFILAUXPRAT AND PD.CDPRODUTO = RC.CDPRODUTO AND RC.NRCARDAPIOTRAB = :NRCARDAPIOTRAB AND RC.NRORG = :NRORG ORDER BY PD.CDARVPROD"
        );
        
        $idreceita = $this->determinaReceitaUtilizada($row);
        if($idreceita) {
            $params = array(
                "P" => array(
                    "CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
                    "CDPRATO"      => $row["CDPRATO"] 
                ),
                "F" => array(
                    "CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
                    "CDPRATO"      => $row["CDPRATO"], 
                    "NRORG"        => $this->environment->getNrOrgTrab()
                ),
                "Q" => array(
                    "CDFILAUXPRAT"      => $row["CDFILAUXPRAT"],
                    "CDPRATO"           => $row["CDPRATO"], 
                    "NRPADRAOQUALIDADE" => $row["NRPADRAOQUALIDADE"]
                ),
                "R" =>  array(
                    "CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
                    "CDPRATO"      => $row["CDPRATO"], 
                    "CDREGIAO"     => $row["CDREGIAO"]
                ),
                "S" =>  array(
                    "CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
                    "CDPRATO"      => $row["CDPRATO"], 
                    "NRORG"        => $this->environment->getNrOrgTrab(), 
                    "CDSERVFILI"   => $row["CDSERVFILI"]
                ),
                "C" => array(
                    "CDFILAUXPRAT"  => $row["CDFILAUXPRAT"],
                    "CDPRATO"       => $row["CDPRATO"],
                    "NRCARDAPIOTRAB"=> $row["NRCARDAPIOTRAB"],
                    "NRORG"         => $this->environment->getNrOrgTrab()
                )
            );
            
            $query = $connection->prepare($consultas[$idreceita]);
            
            $query->execute($params[$idreceita]);
            
            $result = $query->fetchAll(\PDO::FETCH_ASSOC);
            
            return $result;
        } else {
            return array();
        }
        
    }
    
    public function salvaNovoPratoComp(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $compcardbase = $this->compCardBaseExistente($row);
        if(!$compcardbase) {
            $row["IDPRATOEXIBICAO"] = "S";
            $this->salvaNovaCompCardBase($this->buildRowToSaveCompCardBase($row));
            
        } else {
            $row["NRCOMPCARDBASE"] = $compcardbase["NRCOMPCARDBASE"];
        }
        $this->verificaInativaRegraDia($row, $response);
        $this->salvaNovoPratCompCardBase($this->buildRowToSavePratCompCardBase($row), $response);
        if(isset($this->retornos["pratcompcardbase"])){
            $this->salvaNovoRegiaoPratCardBase($this->buildRowsToSaveRegiaoPratCardBase($row));
            $response->addNotification(new Notification("TCO01000_PRATO_SALVO", "M", $this->columnToReturn));
        }
    }
    
    private function compCardBaseExistente($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NRCOMPCARDBASE,
                       NRCARDAPIOBASE,
                       NRDIACARDBASE,
                       NRCONFIGCARDBASE,
                       NRORG
                  FROM COMPCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NRDIACARDBASE = :NRDIACARDBASE 
                   AND NRCONFIGCARDBASE = :NRCONFIGCARDBASE 
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        $params = array(
            "NRCARDAPIOBASE"   => $row["NRCARDAPIOBASE"],
            "NRDIACARDBASE"    => $row["NRDIACARDBASE"],
            "NRCONFIGCARDBASE" => $row["NRCONFIGCARDBASE"],
            "NRORG"            => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function salvaNovaCompCardBase($rowsToSave) {
        $dataSet = new DataSet("compcardbase", $rowsToSave);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "compcardbase");
        $this->crudService->save("compcardbase", $rows);
        $this->retornos["compcardbase"] = $rows[0];
    }
    
    private function buildRowToSaveCompCardBase($row) {
        $rowToSave = array(
            array(
                "NRCARDAPIOBASE"    => $row["NRCARDAPIOBASE"],
                "NRDIACARDBASE"     => $row["NRDIACARDBASE"], 
                "NRCONFIGCARDBASE"  => $row["NRCONFIGCARDBASE"], 
                "__using_unit"      => false, 
                "__new_code_pk"     => "COMPCARDBASE", 
                "__table_pk"        => "NRCOMPCARDBASE", 
                "__new_code_length" => 1, 
                "__cast_number"     => false, 
                "__is_new"          => $row["__is_new"]
            )
        );
        
        return $rowToSave;
    }
    
    private function salvaNovoPratCompCardBase($rowsToSave, $response) {
        if($this->verificaExistenciaPratCompCardBase($rowsToSave[0])) {
            $response->addMessage(new Message('TCO_PRATO_UTILIZADO'));
            return false;
        } else {
            $dataSet = new DataSet("pratcompcardbase", $rowsToSave);
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rows = $this->crudService->getRowsToSave($request, "pratcompcardbase");
            $this->crudService->save("pratcompcardbase", $rows);
            $this->columnToReturn = array(
                "NRCOMPCARDBASE" => $rows[0]["NRCOMPCARDBASE"],
                "NRPRATCOMPCARDBASE" => $rows[0]["NRPRATCOMPCARDBASE"],
                "CDPRATO" => $rows[0]["CDPRATO"],
                "CDFILAUXPRAT" => $rows[0]["CDFILAUXPRAT"],
                "NMPRATO" => $rows[0]["NMPRATO"]
            );
            $this->retornos["pratcompcardbase"] = $rows[0];
        }
    }
    
    private function buildRowToSavePratCompCardBase($row) {
        
        $nrcompcardbase = isset($row["NRCOMPCARDBASE"]) ? $row["NRCOMPCARDBASE"] : $this->retornos["compcardbase"]["NRCOMPCARDBASE"];
        
        $rowToSave = array(
            array(
                "NRCOMPCARDBASE"    => $nrcompcardbase,
                "CDFILAUXPRAT"      => $row["CDFILAUXPRAT"], 
                "CDPRATO"           => $row["CDPRATO"], 
                "NMPRATO"           => $row["NMPRATO"], 
                "IDPRATOEXIBICAO"   => $this->verificaPratosAtribuidos($nrcompcardbase) ? "N" : "S", 
                "__using_unit"      => false, 
                "__new_code_pk"     => "PRATCOMPCARDBASE", 
                "__table_pk"        => "NRPRATCOMPCARDBASE", 
                "__new_code_length" => 1, 
                "__cast_number"     => false, 
                "__is_new"          => $row["__is_new"]
            )
        );
        
        return $rowToSave;
    }
    
    private function verificaPratosAtribuidos($nrcompcardbase) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT COUNT(1) AS OCORRENCIAS
                  FROM PRATCOMPCARDBASE
                 WHERE NRCOMPCARDBASE = :NRCOMPCARDBASE
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCOMPCARDBASE" => $nrcompcardbase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["OCORRENCIAS"] > 0;
    }
    
    private function verificaExistenciaPratCompCardBase($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT COUNT(1) AS OCORRENCIA
                  FROM PRATCOMPCARDBASE
                 WHERE NRCOMPCARDBASE = :NRCOMPCARDBASE 
                   AND CDFILAUXPRAT = :CDFILAUXPRAT 
                   AND CDPRATO = :CDPRATO 
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCOMPCARDBASE" => $row["NRCOMPCARDBASE"],
            "CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
            "CDPRATO"        => $row["CDPRATO"],
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["OCORRENCIA"] > 0;
    }
    
    private function verificaInativaRegraDia($row, $response) {
        $regras = $this->retornaRegraDiaEmUso($row);
        if(sizeof($regras)) {
            $this->inativaRegra($regras);
            $response->addDataSet(new DataSet("adesaoCardBaseInativada", $regras));
            $response->addMessage(new Message("TCO01000_ADESAOCARDBASE_INATIVADA", "M", $regras[0]));
        }
    }
    
    private function retornaRegraDiaEmUso($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NRADESAOCARDBASE,
                       NRCARDAPIOBASE,
                       CDADESAOCARDBASE,
                       NMADESAOCARDBASE,
                       CDGRTIPOPR,
                       NRDIACARDBASE,
                       NRORG,
                       IDATIVO
                  FROM ADESAOCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NRDIACARDBASE = :NRDIACARDBASE
                   AND IDATIVO = 'S'
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE"  => $row["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $row["NRDIACARDBASE"],
            "NRORG"  => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function inativaRegra($regras) {
        $regras[0]["IDATIVO"] = "N";
        $regras[0]["__is_new"] = false;
        $regras[0]["changed"] = true;
        
        $dataSet = new DataSet("adesaocardbase", $regras);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "adesaocardbase");
        $this->crudService->save("adesaocardbase", $rows);
    }
    
    private function salvaNovoRegiaoPratCardBase($rowsToSave) {
        $dataSet = new DataSet("regiaopratcardbase", $rowsToSave);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "regiaopratcardbase");
        $this->crudService->save("regiaopratcardbase", $rows);
        $this->retornos["regiaopratcardbase"] = $rows;
    }
    
    private function buildRowsToSaveRegiaoPratCardBase($row) {
        $rowsToSave = array();
        if($row["CDREGIAO"] === "T"){
            $row["CDREGIAO"] = $this->retornaTodasRegioes($this->retornos["pratcompcardbase"]["NRCOMPCARDBASE"]);
        }
        foreach ($row["CDREGIAO"] as $regiao) {
            array_push($rowsToSave, array(
                "NRPRATCOMPCARDBASE" => $this->retornos["pratcompcardbase"]["NRPRATCOMPCARDBASE"],
                "CDREGIAO"           => $regiao, 
                "__using_unit"       => false, 
                "__new_code_pk"      => "REGIAOPRATCARDBASE", 
                "__table_pk"         => "NRREGIAOPRATCARDBASE", 
                "__new_code_length"  => 1, 
                "__cast_number"      => false, 
                "__is_new"           => $row["__is_new"]
            ));
        };
        return $rowsToSave;
    }
    
    private function retornaTodasRegioes($nrcompcardbase) {
        $connection = $this->entityManager->getConnection();
        $regioes = array();
        $sql = "SELECT CDREGIAO
                  FROM REGIAO
                 WHERE CDREGIAO NOT IN ( SELECT DISTINCT REGIAOPRATCARDBASE.CDREGIAO
                                           FROM REGIAOPRATCARDBASE
                                          INNER JOIN PRATCOMPCARDBASE 
                                             ON PRATCOMPCARDBASE.NRPRATCOMPCARDBASE = REGIAOPRATCARDBASE.NRPRATCOMPCARDBASE 
                                          WHERE PRATCOMPCARDBASE.NRCOMPCARDBASE = :NRCOMPCARDBASE )
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCOMPCARDBASE" => $nrcompcardbase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        foreach ($result as $regiao) {
            array_push($regioes, $regiao["CDREGIAO"]);
        }
        
        return $regioes;
        
    }

}
